@@ -1,6 +1,6 @@
 //////////////////////////////////////////////////////////////////////////////
 //
-// (C) Copyright Ion Gaztanaga 2005-2009. Distributed under the Boost
+// (C) Copyright Ion Gaztanaga 2005-2011. Distributed under the Boost
 // Software License, Version 1.0. (See accompanying file
 // LICENSE_1_0.txt or copy at http://www.boost.org/LICENSE_1_0.txt)
 //
@@ -18,7 +18,7 @@
 #include <boost/interprocess/detail/config_begin.hpp>
 #include <boost/interprocess/detail/workaround.hpp>
 
-#include <boost/pointer_to_other.hpp>
+#include <boost/intrusive/pointer_traits.hpp>
 
 #include <boost/detail/no_exceptions_support.hpp>
 #include <boost/interprocess/detail/type_traits.hpp>
@@ -27,6 +27,7 @@
 #include <boost/interprocess/exceptions.hpp>
 #include <boost/type_traits/make_unsigned.hpp>
 #include <boost/type_traits/alignment_of.hpp>
+#include <boost/intrusive/pointer_traits.hpp>
 #include <cstddef>   //std::size_t
 #include <string>    //char_traits
 #include <new>       //std::nothrow
@@ -90,7 +91,7 @@ struct block_header
       ,  m_num_char((unsigned short)num_char)
       ,  m_value_alignment((unsigned char)value_alignment)
       ,  m_alloc_type_sizeof_char
-         ( (alloc_type << 5u) | 
+         ( (alloc_type << 5u) |
            ((unsigned char)sizeof_char & 0x1F)   )
    {};
 
@@ -129,7 +130,7 @@ struct block_header
 
    template<class CharType>
    CharType *name() const
-   {  
+   { 
       return const_cast<CharType*>(reinterpret_cast<const CharType*>
          (reinterpret_cast<const char*>(this) + name_offset()));
    }
@@ -138,7 +139,7 @@ struct block_header
    {  return m_num_char;   }
 
    size_type name_offset() const
-   { 
+   {
       return this->value_offset() + get_rounded_size(size_type(m_value_bytes), size_type(sizeof_char()));
    }
 
@@ -156,7 +157,7 @@ struct block_header
    bool less_comp(const block_header<size_type> &b) const
    {
       return m_num_char < b.m_num_char ||
-             (m_num_char < b.m_num_char && 
+             (m_num_char < b.m_num_char &&
                std::char_traits<CharType>::compare
                   (name<CharType>(), b.name<CharType>(), m_num_char) < 0);
    }
@@ -174,10 +175,10 @@ struct block_header
    {  return block_header_from_value(value, sizeof(T), ::boost::alignment_of<T>::value);  }
 
    static block_header<size_type> *block_header_from_value(const void *value, std::size_t sz, std::size_t algn)
-   {  
-      block_header * hdr = 
+   { 
+      block_header * hdr =
          const_cast<block_header*>
-            (reinterpret_cast<const block_header*>(reinterpret_cast<const char*>(value) - 
+            (reinterpret_cast<const block_header*>(reinterpret_cast<const char*>(value) -
                get_rounded_size(sizeof(block_header), algn)));
       (void)sz;
       //Some sanity checks
@@ -188,9 +189,9 @@ struct block_header
 
    template<class Header>
    static block_header<size_type> *from_first_header(Header *header)
-   {  
-      block_header<size_type> * hdr = 
-         reinterpret_cast<block_header<size_type>*>(reinterpret_cast<char*>(header) + 
+   { 
+      block_header<size_type> * hdr =
+         reinterpret_cast<block_header<size_type>*>(reinterpret_cast<char*>(header) +
 		 get_rounded_size(size_type(sizeof(Header)), size_type(::boost::alignment_of<block_header<size_type> >::value)));
       //Some sanity checks
       return hdr;
@@ -198,16 +199,16 @@ struct block_header
 
    template<class Header>
    static Header *to_first_header(block_header<size_type> *bheader)
-   {  
-      Header * hdr = 
-         reinterpret_cast<Header*>(reinterpret_cast<char*>(bheader) - 
+   { 
+      Header * hdr =
+         reinterpret_cast<Header*>(reinterpret_cast<char*>(bheader) -
 		 get_rounded_size(size_type(sizeof(Header)), size_type(::boost::alignment_of<block_header<size_type> >::value)));
       //Some sanity checks
       return hdr;
    }
 };
 
-inline void array_construct(void *mem, std::size_t num, ipcdetail::in_place_interface &table)
+inline void array_construct(void *mem, std::size_t num, in_place_interface &table)
 {
    //Try constructors
    std::size_t constructed = 0;
@@ -281,7 +282,7 @@ struct intrusive_value_type_impl
    {
       return const_cast<block_header<size_type>*>
          (reinterpret_cast<const block_header<size_type> *>(reinterpret_cast<const char*>(this) +
-            ::boost::interprocess::ipcdetail::get_rounded_size(size_type(sizeof(*this)), size_type(BlockHdrAlignment))));
+            get_rounded_size(size_type(sizeof(*this)), size_type(BlockHdrAlignment))));
    }
 
    bool operator <(const intrusive_value_type_impl<Hook, CharType, SizeType> & other) const
@@ -310,15 +311,15 @@ template<class CharType>
 class char_ptr_holder
 {
    public:
-   char_ptr_holder(const CharType *name) 
+   char_ptr_holder(const CharType *name)
       : m_name(name)
    {}
 
-   char_ptr_holder(const ipcdetail::anonymous_instance_t *) 
+   char_ptr_holder(const anonymous_instance_t *)
       : m_name(static_cast<CharType*>(0))
    {}
 
-   char_ptr_holder(const ipcdetail::unique_instance_t *) 
+   char_ptr_holder(const unique_instance_t *)
       : m_name(reinterpret_cast<CharType*>(-1))
    {}
 
@@ -329,15 +330,16 @@ class char_ptr_holder
    const CharType *m_name;
 };
 
-//!The key of the the named allocation information index. Stores an offset pointer 
+//!The key of the the named allocation information index. Stores an offset pointer
 //!to a null terminated string and the length of the string to speed up sorting
 template<class CharT, class VoidPointer>
 struct index_key
 {
-   typedef typename boost::
-      pointer_to_other<VoidPointer, const CharT>::type   const_char_ptr_t;
+   typedef typename boost::intrusive::
+      pointer_traits<VoidPointer>::template
+         rebind_pointer<const CharT>::type               const_char_ptr_t;
    typedef CharT                                         char_type;
-   typedef typename std::iterator_traits<const_char_ptr_t>::difference_type difference_type;
+   typedef typename boost::intrusive::pointer_traits<const_char_ptr_t>::difference_type difference_type;
    typedef typename boost::make_unsigned<difference_type>::type size_type;
 
    private:
@@ -354,20 +356,20 @@ struct index_key
    //!Less than function for index ordering
    bool operator < (const index_key & right) const
    {
-      return (m_len < right.m_len) || 
-               (m_len == right.m_len && 
-               std::char_traits<char_type>::compare 
-                  (ipcdetail::get_pointer(mp_str)
-              ,ipcdetail::get_pointer(right.mp_str), m_len) < 0);
+      return (m_len < right.m_len) ||
+               (m_len == right.m_len &&
+               std::char_traits<char_type>::compare
+                  (to_raw_pointer(mp_str)
+              ,to_raw_pointer(right.mp_str), m_len) < 0);
    }
 
    //!Equal to function for index ordering
    bool operator == (const index_key & right) const
    {
-      return   m_len == right.m_len && 
-               std::char_traits<char_type>::compare 
-                  (ipcdetail::get_pointer(mp_str),
-                   ipcdetail::get_pointer(right.mp_str), m_len) == 0;
+      return   m_len == right.m_len &&
+               std::char_traits<char_type>::compare
+                  (to_raw_pointer(mp_str),
+                   to_raw_pointer(right.mp_str), m_len) == 0;
    }
 
    void name(const CharT *name)
@@ -377,7 +379,7 @@ struct index_key
    {  m_len = len; }
 
    const CharT *name() const
-   {  return ipcdetail::get_pointer(mp_str); }
+   {  return to_raw_pointer(mp_str); }
 
    size_type name_length() const
    {  return m_len; }
@@ -393,7 +395,7 @@ struct index_data
    index_data(void *ptr) : m_ptr(ptr){}
 
    void *value() const
-   {  return static_cast<void*>(ipcdetail::get_pointer(m_ptr));  }
+   {  return static_cast<void*>(to_raw_pointer(m_ptr));  }
 };
 
 template<class MemoryAlgorithm>
@@ -405,14 +407,14 @@ struct index_config
 {
    typedef typename MemoryAlgorithm::void_pointer        void_pointer;
    typedef CharT                                         char_type;
-   typedef ipcdetail::index_key<CharT, void_pointer>        key_type;
-   typedef ipcdetail::index_data<void_pointer>              mapped_type;
+   typedef index_key<CharT, void_pointer>        key_type;
+   typedef index_data<void_pointer>              mapped_type;
    typedef typename segment_manager_base_type
       <MemoryAlgorithm>::type                            segment_manager_base;
 
    template<class HeaderBase>
    struct intrusive_value_type
-   {  typedef ipcdetail::intrusive_value_type_impl<HeaderBase, CharT, typename segment_manager_base::size_type>  type; };
+   {  typedef intrusive_value_type_impl<HeaderBase, CharT, typename segment_manager_base::size_type>  type; };
 
    typedef intrusive_compare_key<CharT>            intrusive_compare_key_type;
 };
@@ -464,7 +466,7 @@ class segment_manager_iterator_value_adaptor<Iterator, false>
    const void *value() const
    {
       return reinterpret_cast<block_header<size_type>*>
-         (ipcdetail::get_pointer(m_val->second.m_ptr))->value();
+         (to_raw_pointer(m_val->second.m_ptr))->value();
    }
 
    const typename Iterator::value_type *m_val;
@@ -476,14 +478,14 @@ struct segment_manager_iterator_transform
                          , segment_manager_iterator_value_adaptor<Iterator, intrusive> >
 {
    typedef segment_manager_iterator_value_adaptor<Iterator, intrusive> result_type;
-   
+  
    result_type operator()(const typename Iterator::value_type &arg) const
    {  return result_type(arg); }
 };
 
 }  //namespace ipcdetail {
 
-//These pointers are the ones the user will use to 
+//These pointers are the ones the user will use to
 //indicate previous allocation types
 static const ipcdetail::anonymous_instance_t   * anonymous_instance = 0;
 static const ipcdetail::unique_instance_t      * unique_instance = 0;
