@@ -42,7 +42,7 @@ namespace intrusive {
 //! <b>Static functions</b>:
 //!
 //! <tt>static node_ptr get_next(const_node_ptr n);</tt>
-//! 
+//!
 //! <tt>static void set_next(node_ptr n, node_ptr next);</tt>
 template<class NodeTraits>
 class circular_slist_algorithms
@@ -63,38 +63,38 @@ class circular_slist_algorithms
 
    //! <b>Effects</b>: Constructs an non-used list element, putting the next
    //!   pointer to null:
-   //!  <tt>NodeTraits::get_next(this_node) == 0</tt>
-   //! 
-   //! <b>Complexity</b>: Constant 
-   //! 
+   //!  <tt>NodeTraits::get_next(this_node) == node_ptr()</tt>
+   //!
+   //! <b>Complexity</b>: Constant
+   //!
    //! <b>Throws</b>: Nothing.
    static void init(node_ptr this_node);
 
    //! <b>Requires</b>: this_node must be in a circular list or be an empty circular list.
-   //! 
+   //!
    //! <b>Effects</b>: Returns true is "this_node" is the only node of a circular list:
    //!  or it's a not inserted node:
-   //!  <tt>return !NodeTraits::get_next(this_node) || NodeTraits::get_next(this_node) == this_node</tt>
-   //! 
-   //! <b>Complexity</b>: Constant 
-   //! 
+   //!  <tt>return node_ptr() == NodeTraits::get_next(this_node) || NodeTraits::get_next(this_node) == this_node</tt>
+   //!
+   //! <b>Complexity</b>: Constant
+   //!
    //! <b>Throws</b>: Nothing.
    static bool unique(const_node_ptr this_node);
 
    //! <b>Effects</b>: Returns true is "this_node" has the same state as
    //!  if it was inited using "init(node_ptr)"
-   //! 
-   //! <b>Complexity</b>: Constant 
-   //! 
+   //!
+   //! <b>Complexity</b>: Constant
+   //!
    //! <b>Throws</b>: Nothing.
    static bool inited(const_node_ptr this_node);
 
    //! <b>Requires</b>: prev_node must be in a circular list or be an empty circular list.
-   //! 
+   //!
    //! <b>Effects</b>: Unlinks the next node of prev_node from the circular list.
-   //! 
-   //! <b>Complexity</b>: Constant 
-   //! 
+   //!
+   //! <b>Complexity</b>: Constant
+   //!
    //! <b>Throws</b>: Nothing.
    static void unlink_after(node_ptr prev_node);
 
@@ -103,28 +103,28 @@ class circular_slist_algorithms
    //!
    //! <b>Effects</b>: Unlinks the range (prev_node, last_node) from the circular list.
    //!
-   //! <b>Complexity</b>: Constant 
+   //! <b>Complexity</b>: Constant
    //!
    //! <b>Throws</b>: Nothing.
    static void unlink_after(node_ptr prev_node, node_ptr last_node);
 
    //! <b>Requires</b>: prev_node must be a node of a circular list.
-   //! 
+   //!
    //! <b>Effects</b>: Links this_node after prev_node in the circular list.
-   //! 
-   //! <b>Complexity</b>: Constant 
-   //! 
+   //!
+   //! <b>Complexity</b>: Constant
+   //!
    //! <b>Throws</b>: Nothing.
    static void link_after(node_ptr prev_node, node_ptr this_node);
 
    //! <b>Requires</b>: b and e must be nodes of the same circular list or an empty range.
    //!   and p must be a node of a different circular list.
-   //! 
+   //!
    //! <b>Effects</b>: Removes the nodes from (b, e] range from their circular list and inserts
    //!   them after p in p's circular list.
-   //! 
-   //! <b>Complexity</b>: Constant 
-   //! 
+   //!
+   //! <b>Complexity</b>: Constant
+   //!
    //! <b>Throws</b>: Nothing.
    static void transfer_after(node_ptr p, node_ptr b, node_ptr e);
 
@@ -133,55 +133,55 @@ class circular_slist_algorithms
    //! <b>Effects</b>: Constructs an empty list, making this_node the only
    //!   node of the circular list:
    //!  <tt>NodeTraits::get_next(this_node) == this_node</tt>.
-   //! 
-   //! <b>Complexity</b>: Constant 
-   //! 
+   //!
+   //! <b>Complexity</b>: Constant
+   //!
    //! <b>Throws</b>: Nothing.
-   static void init_header(node_ptr this_node)
-   {  NodeTraits::set_next(this_node, this_node);  } 
+   static void init_header(const node_ptr &this_node)
+   {  NodeTraits::set_next(this_node, this_node);  }
 
    //! <b>Requires</b>: this_node and prev_init_node must be in the same circular list.
-   //! 
+   //!
    //! <b>Effects</b>: Returns the previous node of this_node in the circular list starting.
    //!   the search from prev_init_node. The first node checked for equality
    //!   is NodeTraits::get_next(prev_init_node).
-   //! 
+   //!
    //! <b>Complexity</b>: Linear to the number of elements between prev_init_node and this_node.
-   //! 
+   //!
    //! <b>Throws</b>: Nothing.
-   static node_ptr get_previous_node(node_ptr prev_init_node, node_ptr this_node)
+   static node_ptr get_previous_node(const node_ptr &prev_init_node, const node_ptr &this_node)
    {  return base_t::get_previous_node(prev_init_node, this_node);   }
 
    //! <b>Requires</b>: this_node must be in a circular list or be an empty circular list.
-   //! 
+   //!
    //! <b>Effects</b>: Returns the previous node of this_node in the circular list.
-   //! 
+   //!
    //! <b>Complexity</b>: Linear to the number of elements in the circular list.
-   //! 
+   //!
    //! <b>Throws</b>: Nothing.
-   static node_ptr get_previous_node(node_ptr this_node)
+   static node_ptr get_previous_node(const node_ptr & this_node)
    {  return base_t::get_previous_node(this_node, this_node); }
 
    //! <b>Requires</b>: this_node must be in a circular list or be an empty circular list.
-   //! 
+   //!
    //! <b>Effects</b>: Returns the previous node of the previous node of this_node in the circular list.
-   //! 
+   //!
    //! <b>Complexity</b>: Linear to the number of elements in the circular list.
-   //! 
+   //!
    //! <b>Throws</b>: Nothing.
-   static node_ptr get_previous_previous_node(node_ptr this_node)
+   static node_ptr get_previous_previous_node(const node_ptr & this_node)
    {  return get_previous_previous_node(this_node, this_node); }
 
    //! <b>Requires</b>: this_node and prev_prev_init_node must be in the same circular list.
-   //! 
+   //!
    //! <b>Effects</b>: Returns the previous node of the previous node of this_node in the
    //!   circular list starting. the search from prev_init_node. The first node checked
    //!   for equality is NodeTraits::get_next((NodeTraits::get_next(prev_prev_init_node)).
-   //! 
+   //!
    //! <b>Complexity</b>: Linear to the number of elements in the circular list.
-   //! 
+   //!
    //! <b>Throws</b>: Nothing.
-   static node_ptr get_previous_previous_node(node_ptr prev_prev_init_node, node_ptr this_node)
+   static node_ptr get_previous_previous_node(const node_ptr & prev_prev_init_node, const node_ptr & this_node)
    {
       node_ptr p = prev_prev_init_node;
       node_ptr p_next = NodeTraits::get_next(p);
@@ -195,14 +195,14 @@ class circular_slist_algorithms
    }
 
    //! <b>Requires</b>: this_node must be in a circular list or be an empty circular list.
-   //! 
+   //!
    //! <b>Effects</b>: Returns the number of nodes in a circular list. If the circular list
    //!  is empty, returns 1.
-   //! 
-   //! <b>Complexity</b>: Linear 
-   //! 
+   //!
+   //! <b>Complexity</b>: Linear
+   //!
    //! <b>Throws</b>: Nothing.
-   static std::size_t count(const_node_ptr this_node) 
+   static std::size_t count(const const_node_ptr & this_node)
    {
       std::size_t result = 0;
       const_node_ptr p = this_node;
@@ -214,39 +214,39 @@ class circular_slist_algorithms
    }
 
    //! <b>Requires</b>: this_node must be in a circular list, be an empty circular list or be inited.
-   //! 
+   //!
    //! <b>Effects</b>: Unlinks the node from the circular list.
-   //! 
-   //! <b>Complexity</b>: Linear to the number of elements in the circular list 
-   //! 
+   //!
+   //! <b>Complexity</b>: Linear to the number of elements in the circular list
+   //!
    //! <b>Throws</b>: Nothing.
-   static void unlink(node_ptr this_node)
+   static void unlink(const node_ptr & this_node)
    {
       if(NodeTraits::get_next(this_node))
          base_t::unlink_after(get_previous_node(this_node));
    }
 
    //! <b>Requires</b>: nxt_node must be a node of a circular list.
-   //! 
+   //!
    //! <b>Effects</b>: Links this_node before nxt_node in the circular list.
-   //! 
-   //! <b>Complexity</b>: Linear to the number of elements in the circular list. 
-   //! 
+   //!
+   //! <b>Complexity</b>: Linear to the number of elements in the circular list.
+   //!
    //! <b>Throws</b>: Nothing.
-   static void link_before (node_ptr nxt_node, node_ptr this_node)
+   static void link_before (const node_ptr & nxt_node, const node_ptr & this_node)
    {  base_t::link_after(get_previous_node(nxt_node), this_node);   }
 
    //! <b>Requires</b>: this_node and other_node must be nodes inserted
    //!  in circular lists or be empty circular lists.
-   //! 
+   //!
    //! <b>Effects</b>: Swaps the position of the nodes: this_node is inserted in
    //!   other_nodes position in the second circular list and the other_node is inserted
    //!   in this_node's position in the first circular list.
-   //! 
-   //! <b>Complexity</b>: Linear to number of elements of both lists 
-   //! 
+   //!
+   //! <b>Complexity</b>: Linear to number of elements of both lists
+   //!
    //! <b>Throws</b>: Nothing.
-   static void swap_nodes(node_ptr this_node, node_ptr other_node)
+   static void swap_nodes(const node_ptr & this_node, const node_ptr & other_node)
    {
       if (other_node == this_node)
          return;
@@ -279,14 +279,14 @@ class circular_slist_algorithms
       }
    }
 
-   //! <b>Effects</b>: Reverses the order of elements in the list. 
-   //! 
+   //! <b>Effects</b>: Reverses the order of elements in the list.
+   //!
    //! <b>Throws</b>: Nothing.
-   //! 
+   //!
    //! <b>Complexity</b>: This function is linear to the contained elements.
-   static void reverse(node_ptr p)
+   static void reverse(const node_ptr & p)
    {
-      node_ptr i = NodeTraits::get_next(p), e(p); 
+      node_ptr i = NodeTraits::get_next(p), e(p);
       for (;;) {
          node_ptr nxt(NodeTraits::get_next(i));
          if (nxt == e)
@@ -301,20 +301,20 @@ class circular_slist_algorithms
    //!   Null if n leads to no movement.
    //!
    //! <b>Throws</b>: Nothing.
-   //! 
+   //!
    //! <b>Complexity</b>: Linear to the number of elements plus the number moved positions.
-   static node_ptr move_backwards(node_ptr p, std::size_t n)
+   static node_ptr move_backwards(const node_ptr & p, std::size_t n)
    {
       //Null shift, nothing to do
-      if(!n) return node_ptr(0);
+      if(!n) return node_ptr();
       node_ptr first  = NodeTraits::get_next(p);
 
       //count() == 1 or 2, nothing to do
       if(NodeTraits::get_next(first) == p)
-         return node_ptr(0);
+         return node_ptr();
 
       bool end_found = false;
-      node_ptr new_last(0);
+      node_ptr new_last = node_ptr();
 
       //Now find the new last node according to the shift count.
       //If we find p before finding the new last node
@@ -327,7 +327,7 @@ class circular_slist_algorithms
             //Shortcut the shift with the modulo of the size of the list
             n %= i;
             if(!n)
-               return node_ptr(0);
+               return node_ptr();
             i = 0;
             //Unlink p and continue the new first node search
             first = NodeTraits::get_next(p);
@@ -348,21 +348,21 @@ class circular_slist_algorithms
    }
 
    //! <b>Effects</b>: Moves the node p n positions towards the beginning of the list.
-   //! 
+   //!
    //! <b>Returns</b>: The previous node of p after the function if there has been any movement,
    //!   Null if n leads equals to no movement.
-   //! 
+   //!
    //! <b>Throws</b>: Nothing.
-   //! 
+   //!
    //! <b>Complexity</b>: Linear to the number of elements plus the number moved positions.
-   static node_ptr move_forward(node_ptr p, std::size_t n)
+   static node_ptr move_forward(const node_ptr & p, std::size_t n)
    {
       //Null shift, nothing to do
-      if(!n) return node_ptr(0);
+      if(!n) return node_ptr();
       node_ptr first  = node_traits::get_next(p);
 
       //count() == 1 or 2, nothing to do
-      if(node_traits::get_next(first) == p) return node_ptr(0);
+      if(node_traits::get_next(first) == p) return node_ptr();
 
       //Iterate until p is found to know where the current last node is.
       //If the shift count is less than the size of the list, we can also obtain
@@ -381,8 +381,8 @@ class circular_slist_algorithms
          //Shortcut the shift with the modulo of the size of the list
          std::size_t new_before_last_pos = (distance - (n % distance))% distance;
          //If the shift is a multiple of the size there is nothing to do
-         if(!new_before_last_pos)   return node_ptr(0);
-         
+         if(!new_before_last_pos)   return node_ptr();
+        
          for( new_last = p
             ; new_before_last_pos--
             ; new_last = node_traits::get_next(new_last)){
@@ -397,8 +397,8 @@ class circular_slist_algorithms
    }
 };
 
-} //namespace intrusive 
-} //namespace boost 
+} //namespace intrusive
+} //namespace boost
 
 #include <boost/intrusive/detail/config_end.hpp>
 
