@@ -16,6 +16,7 @@
 #include <boost/preprocessor/repetition/enum_params.hpp>
 #include <boost/preprocessor/repetition/enum_trailing_params.hpp>
 #include <boost/preprocessor/repetition/enum_binary_params.hpp>
+#include <boost/preprocessor/repetition/repeat.hpp>
 #include <boost/preprocessor/repetition/repeat_from_to.hpp>
 #include <boost/preprocessor/iteration/local.hpp>
 #include <boost/mpl/if.hpp>
@@ -32,17 +33,21 @@
 #include <boost/utility/addressof.hpp>
 #include <boost/utility/result_of.hpp>
 #include <boost/utility/enable_if.hpp>
-#include <boost/proto/repeat.hpp>
+
+#if defined(_MSC_VER) && (_MSC_VER >= 1020)
+# pragma warning(push)
+# pragma warning(disable : 4714) // function 'xxx' marked as __forceinline not inlined
+#endif
 
 #ifndef BOOST_NO_DECLTYPE
 # define BOOST_PROTO_DECLTYPE_(EXPR, TYPE) typedef decltype(EXPR) TYPE;
 #else
 # define BOOST_PROTO_DECLTYPE_NESTED_TYPEDEF_TPL_(NESTED, EXPR)                                     \
     BOOST_TYPEOF_NESTED_TYPEDEF_TPL(BOOST_PP_CAT(nested_and_hidden_, NESTED), EXPR)                 \
-    static int const sz = sizeof(boost::proto::detail::check_reference(EXPR));                      \
+    static int const BOOST_PP_CAT(sz, NESTED) = sizeof(boost::proto::detail::check_reference(EXPR));\
     struct NESTED                                                                                   \
       : boost::mpl::if_c<                                                                           \
-            1==sz                                                                                   \
+            1 == BOOST_PP_CAT(sz, NESTED)                                                           \
           , typename BOOST_PP_CAT(nested_and_hidden_, NESTED)::type &                               \
           , typename BOOST_PP_CAT(nested_and_hidden_, NESTED)::type                                 \
         >                                                                                           \
@@ -228,7 +233,7 @@ namespace boost { namespace proto
             template<typename T>
             struct has_get_pointer
             {
-                BOOST_STATIC_CONSTANT(bool, value = sizeof(void *) == sizeof(get_pointer(make<T &>())));
+                static const bool value = sizeof(void *) == sizeof(get_pointer(make<T &>()));
                 typedef mpl::bool_<value> type;
             };
         }
@@ -245,20 +250,8 @@ namespace boost { namespace proto
             typedef U type;
         };
 
-        #define BOOST_PP_LOCAL_MACRO(N)                                                             \
-        template<typename T, typename U BOOST_PP_ENUM_TRAILING_PARAMS(N, typename A)>               \
-        struct classtypeof<T (U::*)(BOOST_PP_ENUM_PARAMS(N, A))>                                    \
-        {                                                                                           \
-            typedef U type;                                                                         \
-        };                                                                                          \
-        template<typename T, typename U BOOST_PP_ENUM_TRAILING_PARAMS(N, typename A)>               \
-        struct classtypeof<T (U::*)(BOOST_PP_ENUM_PARAMS(N, A)) const>                              \
-        {                                                                                           \
-            typedef U type;                                                                         \
-        };                                                                                          \
-        /**/
-        #define BOOST_PP_LOCAL_LIMITS (0, BOOST_PROTO_MAX_ARITY)
-        #include BOOST_PP_LOCAL_ITERATE()
+        // Other specializations are generated by the preprocessor
+        #include <boost/proto/detail/classtypeof.hpp>
 
         ////////////////////////////////////////////////////////////////////////////////////////////
         template<typename T>
@@ -342,8 +335,8 @@ namespace boost { namespace proto
             template<typename T, typename U, typename V>
             struct result_of_<T U::*(V), typename enable_if_c<is_member_object_pointer<T U::*>::value>::type>
             {
-                BOOST_STATIC_CONSTANT(bool, is_V_a_smart_ptr = 2 == sizeof(test_V_is_a_U<U>(&lvalue(make<V>()))));
-                BOOST_STATIC_CONSTANT(bool, is_ptr_to_const = 2 == sizeof(test_ptr_to_const(BOOST_PROTO_GET_POINTER(U, make<V>()))));
+                static const bool is_V_a_smart_ptr = 2 == sizeof(test_V_is_a_U<U>(&lvalue(make<V>())));
+                static const bool is_ptr_to_const = 2 == sizeof(test_ptr_to_const(BOOST_PROTO_GET_POINTER(U, make<V>())));
 
                 // If V is not a U, then it is a (smart) pointer and we can always return an lvalue.
                 // Otherwise, we can only return an lvalue if we are given one.
@@ -499,17 +492,8 @@ namespace boost { namespace proto
                 return (BOOST_PROTO_GET_POINTER(V, obj) ->* pmf)();
             }
 
-            #define BOOST_PROTO_LOCAL_MACRO(N, typename_A, A_const_ref, A_const_ref_a, a)           \
-            template<typename_A(N)>                                                                 \
-            result_type operator()(A_const_ref_a(N)) const                                          \
-            {                                                                                       \
-                BOOST_PROTO_USE_GET_POINTER();                                                      \
-                return (BOOST_PROTO_GET_POINTER(V, obj) ->* pmf)(a(N));                             \
-            }                                                                                       \
-            /**/
-            #define BOOST_PROTO_LOCAL_a BOOST_PROTO_a
-            #define BOOST_PROTO_LOCAL_LIMITS (1, BOOST_PROTO_MAX_ARITY)
-            #include BOOST_PROTO_LOCAL_ITERATE()
+            // Other overloads generated by the preprocessor
+            #include <boost/proto/detail/memfun_funop.hpp>
 
         private:
             T obj;
@@ -519,4 +503,8 @@ namespace boost { namespace proto
     } // namespace detail
 }}
 
+#if defined(_MSC_VER) && (_MSC_VER >= 1020)
+# pragma warning(pop)
+#endif
+
 #endif
