@@ -1,4 +1,5 @@
-//  Copyright (c) 2001-2010 Hartmut Kaiser
+//  Copyright (c) 2001-2011 Hartmut Kaiser
+//  Copyright (c)      2010 Bryce Lelbach
 //
 //  Distributed under the Boost Software License, Version 1.0. (See accompanying
 //  file LICENSE_1_0.txt or copy at http://www.boost.org/LICENSE_1_0.txt)
@@ -23,12 +24,14 @@
 #include <boost/spirit/home/karma/auxiliary/lazy.hpp>
 #include <boost/spirit/home/karma/detail/get_casetag.hpp>
 #include <boost/spirit/home/karma/detail/generate_to.hpp>
+#include <boost/spirit/home/karma/detail/enable_lit.hpp>
 #include <boost/fusion/include/at.hpp>
 #include <boost/fusion/include/vector.hpp>
 #include <boost/fusion/include/cons.hpp>
 #include <boost/mpl/if.hpp>
 #include <boost/mpl/assert.hpp>
 #include <boost/mpl/bool.hpp>
+#include <boost/utility/enable_if.hpp>
 #include <string>
 
 ///////////////////////////////////////////////////////////////////////////////
@@ -50,6 +53,12 @@ namespace boost { namespace spirit
         >
     > : mpl::true_ {};
 
+    template <typename A0>
+    struct use_terminal<karma::domain
+          , terminal_ex<tag::lit, fusion::vector1<A0> > // enables lit('x')
+          , typename enable_if<traits::is_char<A0> >::type>
+      : mpl::true_ {};
+
     template <typename CharEncoding, typename A0, typename A1>
     struct use_terminal<karma::domain
       , terminal_ex<
@@ -80,13 +89,15 @@ namespace boost { namespace spirit
     template <>
     struct use_terminal<karma::domain, wchar_t[2]>      // enables L"x"
       : mpl::true_ {};
-
 }}
 
 ///////////////////////////////////////////////////////////////////////////////
 namespace boost { namespace spirit { namespace karma
 {
+#ifndef BOOST_SPIRIT_NO_PREDEFINED_TERMINALS
     using spirit::lit;    // lit('x') is equivalent to 'x'
+#endif
+    using spirit::lit_type;
 
     ///////////////////////////////////////////////////////////////////////////
     //
@@ -242,13 +253,19 @@ namespace boost { namespace spirit { namespace karma
 
     ///////////////////////////////////////////////////////////////////////////
     // character set generator
-    template <typename CharEncoding, typename Tag>
+    template <typename CharEncoding, typename Tag, bool no_attribute>
     struct char_set
-      : char_generator<char_set<CharEncoding, Tag>, CharEncoding, Tag>
+      : char_generator<char_set<CharEncoding, Tag, no_attribute>
+          , CharEncoding, Tag>
     {
         typedef typename CharEncoding::char_type char_type;
         typedef CharEncoding char_encoding;
 
+        template <typename Context, typename Unused>
+        struct attribute
+          : mpl::if_c<no_attribute, unused_type, char_type>
+        {};
+
         template <typename String>
         char_set(String const& str)
         {
@@ -390,45 +407,65 @@ namespace boost { namespace spirit { namespace karma
         }
     };
 
-    // char_(...)
+    ///////////////////////////////////////////////////////////////////////////
+    namespace detail
+    {
+        template <typename CharEncoding, typename Modifiers, typename A0
+          , bool no_attribute>
+        struct make_char_direct
+        {
+            static bool const lower =
+                has_modifier<Modifiers, tag::char_code_base<tag::lower> >::value;
+            static bool const upper =
+                has_modifier<Modifiers, tag::char_code_base<tag::upper> >::value;
+
+            typedef typename spirit::detail::get_encoding_with_case<
+                Modifiers, CharEncoding, lower || upper>::type encoding;
+            typedef typename detail::get_casetag<
+                Modifiers, lower || upper>::type tag;
+
+            typedef typename mpl::if_<
+                traits::is_string<A0>
+              , char_set<encoding, tag, no_attribute>
+              , literal_char<encoding, tag, no_attribute>
+            >::type result_type;
+
+            template <typename Terminal>
+            result_type operator()(Terminal const& term, unused_type) const
+            {
+                return result_type(fusion::at_c<0>(term.args));
+            }
+        };
+    }
+
+    // char_(...), lit(...)
     template <typename CharEncoding, typename Modifiers, typename A0>
     struct make_primitive<
-        terminal_ex<
-            tag::char_code<tag::char_, CharEncoding>
-          , fusion::vector1<A0>
-        >
-      , Modifiers>
-    {
-        static bool const lower =
-            has_modifier<Modifiers, tag::char_code_base<tag::lower> >::value;
-        static bool const upper =
-            has_modifier<Modifiers, tag::char_code_base<tag::upper> >::value;
+            terminal_ex<
+                tag::char_code<tag::char_, CharEncoding>
+              , fusion::vector1<A0> >
+          , Modifiers>
+      : detail::make_char_direct<CharEncoding, Modifiers, A0, false>
+    {};
 
-        typedef typename spirit::detail::get_encoding_with_case<
-            Modifiers, CharEncoding, lower || upper>::type encoding;
-        typedef typename detail::get_casetag<
-            Modifiers, lower || upper>::type tag;
-
-        typedef typename mpl::if_<
-            traits::is_string<A0>
-          , char_set<encoding, tag>
-          , literal_char<encoding, tag, false>
-        >::type result_type;
-
-        template <typename Terminal>
-        result_type operator()(Terminal const& term, unused_type) const
-        {
-            return result_type(fusion::at_c<0>(term.args));
-        }
-    };
+    template <typename Modifiers, typename A0>
+    struct make_primitive<
+            terminal_ex<tag::lit, fusion::vector1<A0> >
+          , Modifiers
+          , typename enable_if<traits::is_char<A0> >::type>
+      : detail::make_char_direct<
+            typename traits::char_encoding_from_char<
+                typename traits::char_type_of<A0>::type>::type
+          , Modifiers, A0, true>
+    {};
 
+    ///////////////////////////////////////////////////////////////////////////
     // char_("x")
     template <typename CharEncoding, typename Modifiers, typename Char>
     struct make_primitive<
         terminal_ex<
             tag::char_code<tag::char_, CharEncoding>
-          , fusion::vector1<Char(&)[2]> // For single char strings
-        >
+          , fusion::vector1<Char(&)[2]> > // For single char strings
       , Modifiers>
     {
         static bool const lower =
@@ -450,6 +487,7 @@ namespace boost { namespace spirit { namespace karma
         }
     };
 
+    ///////////////////////////////////////////////////////////////////////////
     // char_('a', 'z')
     template <typename CharEncoding, typename Modifiers, typename A0, typename A1>
     struct make_primitive<
@@ -478,6 +516,32 @@ namespace boost { namespace spirit { namespace karma
         }
     };
 
+    template <typename CharEncoding, typename Modifiers, typename Char>
+    struct make_primitive<
+        terminal_ex<
+            tag::char_code<tag::char_, CharEncoding>
+          , fusion::vector2<Char(&)[2], Char(&)[2]> // For single char strings
+        >
+      , Modifiers>
+    {
+        static bool const lower =
+            has_modifier<Modifiers, tag::char_code_base<tag::lower> >::value;
+        static bool const upper =
+            has_modifier<Modifiers, tag::char_code_base<tag::upper> >::value;
+
+        typedef char_range<
+            typename spirit::detail::get_encoding_with_case<
+                Modifiers, CharEncoding, lower || upper>::type
+          , typename detail::get_casetag<Modifiers, lower || upper>::type
+        > result_type;
+
+        template <typename Terminal>
+        result_type operator()(Terminal const& term, unused_type) const
+        {
+            return result_type(fusion::at_c<0>(term.args)[0]
+              , fusion::at_c<1>(term.args)[0]);
+        }
+    };
 }}}   // namespace boost::spirit::karma
 
 #endif
