@@ -5,7 +5,7 @@
 
     http://www.boost.org/
 
-    Copyright (c) 2001-2010 Hartmut Kaiser. Distributed under the Boost
+    Copyright (c) 2001-2012 Hartmut Kaiser. Distributed under the Boost
     Software License, Version 1.0. (See accompanying file
     LICENSE_1_0.txt or copy at http://www.boost.org/LICENSE_1_0.txt)
 =============================================================================*/
@@ -27,7 +27,7 @@
 #if !defined(BOOST_WAVE_TOKEN_IDS_DEFINED)
 #define BOOST_WAVE_TOKEN_IDS_DEFINED
 
-#if (defined (__FreeBSD__) || defined (__DragonFly__)) && defined (T_DIVIDE) 
+#if (defined (__FreeBSD__) || defined (__DragonFly__) || defined (__OpenBSD__)) && defined (T_DIVIDE) 
 #undef T_DIVIDE
 #endif
 
@@ -44,30 +44,31 @@ namespace wave {
 ///////////////////////////////////////////////////////////////////////////////
 //  the token_category helps to classify the different token types 
 enum token_category {
-    IdentifierTokenType         = 0x10000000,
-    ParameterTokenType          = 0x11000000,
-    ExtParameterTokenType       = 0x11100000,
-    KeywordTokenType            = 0x20000000,
-    OperatorTokenType           = 0x30000000,
-    LiteralTokenType            = 0x40000000,
-    IntegerLiteralTokenType     = 0x41000000,
-    FloatingLiteralTokenType    = 0x42000000,
-    StringLiteralTokenType      = 0x43000000,
-    CharacterLiteralTokenType   = 0x44000000,
-    BoolLiteralTokenType        = 0x45000000,
-    PPTokenType                 = 0x50000000,
-    PPConditionalTokenType      = 0x50800000,
+    IdentifierTokenType         = 0x10080000,
+    ParameterTokenType          = 0x11080000,
+    ExtParameterTokenType       = 0x11180000,
+    KeywordTokenType            = 0x20080000,
+    OperatorTokenType           = 0x30080000,
+    LiteralTokenType            = 0x40080000,
+    IntegerLiteralTokenType     = 0x41080000,
+    FloatingLiteralTokenType    = 0x42080000,
+    StringLiteralTokenType      = 0x43080000,
+    CharacterLiteralTokenType   = 0x44080000,
+    BoolLiteralTokenType        = 0x45080000,
+    PPTokenType                 = 0x50080000,
+    PPConditionalTokenType      = 0x50880000,
 
     UnknownTokenType            = 0xA0000000,
     EOLTokenType                = 0xB0000000,
     EOFTokenType                = 0xC0000000,
     WhiteSpaceTokenType         = 0xD0000000,
-    InternalTokenType           = 0xE0000000,
-    
+    InternalTokenType           = 0xE0080000,
+
     TokenTypeMask               = 0xFF000000,
     AltTokenType                = 0x00100000,
     TriGraphTokenType           = 0x00200000,
     AltExtTokenType             = 0x00500000,   // and, bit_and etc.
+    PPTokenFlag                 = 0x00080000,   // these are 'real' pp-tokens
     ExtTokenTypeMask            = 0xFFF00000,
     ExtTokenOnlyMask            = 0x00F00000,
     TokenValueMask              = 0x000FFFFF,
@@ -262,7 +263,7 @@ enum token_id {
     T_EOF          = TOKEN_FROM_ID(401, EOFTokenType),      // end of file reached
     T_EOI          = TOKEN_FROM_ID(402, EOFTokenType),      // end of input reached
     T_PP_NUMBER    = TOKEN_FROM_ID(403, InternalTokenType),
-    
+
 // MS extensions
     T_MSEXT_INT8   = TOKEN_FROM_ID(404, KeywordTokenType),
     T_MSEXT_INT16  = TOKEN_FROM_ID(405, KeywordTokenType),
@@ -286,8 +287,21 @@ enum token_id {
 // import is needed to be a keyword for the C++ module Standards proposal
     T_IMPORT       = TOKEN_FROM_ID(421, KeywordTokenType),
 
+// C++0x keywords
+    T_ALIGNAS      = TOKEN_FROM_ID(422, KeywordTokenType),
+    T_ALIGNOF      = TOKEN_FROM_ID(423, KeywordTokenType),
+    T_CHAR16_T     = TOKEN_FROM_ID(424, KeywordTokenType),
+    T_CHAR32_T     = TOKEN_FROM_ID(425, KeywordTokenType),
+    T_CONSTEXPR    = TOKEN_FROM_ID(426, KeywordTokenType),
+    T_DECLTYPE     = TOKEN_FROM_ID(427, KeywordTokenType),
+    T_NOEXCEPT     = TOKEN_FROM_ID(428, KeywordTokenType),
+    T_NULLPTR      = TOKEN_FROM_ID(429, KeywordTokenType),
+    T_STATICASSERT = TOKEN_FROM_ID(430, KeywordTokenType),
+    T_THREADLOCAL  = TOKEN_FROM_ID(431, KeywordTokenType),
+    T_RAWSTRINGLIT = TOKEN_FROM_ID(432, StringLiteralTokenType),
+
     T_LAST_TOKEN_ID,
-    T_LAST_TOKEN = ID_FROM_TOKEN(T_LAST_TOKEN_ID),
+    T_LAST_TOKEN = ID_FROM_TOKEN(T_LAST_TOKEN_ID & ~PPTokenFlag),
 
 // pseudo tokens to help streamlining macro replacement, these should not 
 // returned from the lexer nor should these be returned from the pp-iterator
@@ -304,11 +318,15 @@ enum token_id {
 #define TOKEN_FROM_ID(id, cat)   boost::wave::token_id((id) | (cat))
 
 #undef ID_FROM_TOKEN
-#define ID_FROM_TOKEN(tok)       ((tok) & ~boost::wave::TokenTypeMask)
+#define ID_FROM_TOKEN(tok)                                                    \
+    boost::wave::token_id((tok) &                                             \
+        ~(boost::wave::TokenTypeMask|boost::wave::PPTokenFlag))               \
+    /**/
 
 #undef BASEID_FROM_TOKEN
 #define BASEID_FROM_TOKEN(tok)                                                \
-    boost::wave::token_id(((tok) & ~boost::wave::ExtTokenTypeMask))           \
+    boost::wave::token_id((tok) &                                             \
+         ~(boost::wave::ExtTokenTypeMask|boost::wave::PPTokenFlag))           \
     /**/
 #define BASE_TOKEN(tok)                                                       \
     boost::wave::token_id((tok) & boost::wave::MainTokenMask)                 \
@@ -316,12 +334,25 @@ enum token_id {
 #define CATEGORY_FROM_TOKEN(tok) ((tok) & boost::wave::TokenTypeMask)
 #define EXTCATEGORY_FROM_TOKEN(tok) ((tok) & boost::wave::ExtTokenTypeMask)
 #define IS_CATEGORY(tok, cat)                                                 \
-    ((CATEGORY_FROM_TOKEN(tok) == (cat)) ? true : false)                      \
+    ((CATEGORY_FROM_TOKEN(tok) == CATEGORY_FROM_TOKEN(cat)) ? true : false)   \
     /**/
 #define IS_EXTCATEGORY(tok, cat)                                              \
-    ((EXTCATEGORY_FROM_TOKEN(tok) == (cat)) ? true : false)                   \
+    ((EXTCATEGORY_FROM_TOKEN(tok) == EXTCATEGORY_FROM_TOKEN(cat)) ? true : false) \
     /**/
 
+///////////////////////////////////////////////////////////////////////////////
+// verify whether the given token(-id) represents a valid pp_token
+inline bool is_pp_token(boost::wave::token_id id)
+{
+    return (id & boost::wave::PPTokenFlag) ? true : false;
+}
+
+template <typename TokenT>
+inline bool is_pp_token(TokenT const& tok)
+{
+    return is_pp_token(boost::wave::token_id(tok));
+}
+
 ///////////////////////////////////////////////////////////////////////////////
 //  return a token name
 BOOST_WAVE_DECL 
