@@ -11,7 +11,7 @@
 
 //  See http://www.boost.org/libs/config for most recent version.
 //
-//  Revision $Id: config_info.cpp 50531 2009-01-10 11:18:42Z johnmaddock $
+//  Revision $Id: config_info.cpp 78478 2012-05-15 11:57:21Z johnmaddock $
 //
 
 #include <boost/config.hpp>
@@ -27,6 +27,10 @@
 #include <unistd.h>
 #endif
 
+#if defined(__MINGW32__)
+#  include <_mingw.h>
+#endif
+
 static unsigned int indent = 4;
 static unsigned int width = 40;
 
@@ -154,9 +158,11 @@ void print_compiler_macros()
   // GNUC options:
   PRINT_MACRO(__GNUC__);
   PRINT_MACRO(__GNUC_MINOR__);
+  PRINT_MACRO(__GNUC_PATCHLEVEL__);
   PRINT_MACRO(__STDC_VERSION__);
   PRINT_MACRO(__GNUG__);
   PRINT_MACRO(__STRICT_ANSI__);
+  PRINT_MACRO(__GXX_EXPERIMENTAL_CXX0X__);
   PRINT_MACRO(__VERSION__);
   PRINT_MACRO(__OPTIMIZE__);
   PRINT_MACRO(__CHAR_UNSIGNED__);
@@ -288,6 +294,22 @@ void print_compiler_macros()
   PRINT_MACRO(_PGO_INSTRUMENT);
   PRINT_MACRO(__QMSPP_); 
 
+  // Cray options:
+  PRINT_MACRO(_CRAYC); 
+  PRINT_MACRO(_RELEASE); 
+  PRINT_MACRO(cray); 
+  PRINT_MACRO(CRAY); 
+  PRINT_MACRO(CRAY1); 
+  PRINT_MACRO(_CRAY1); 
+  PRINT_MACRO(_CRAYMPP); 
+  PRINT_MACRO(_CRAYT3E); 
+  PRINT_MACRO(_CRAYIEEE); 
+  PRINT_MACRO(_ADDR32); 
+  PRINT_MACRO(_ADDR64); 
+  PRINT_MACRO(_LD64); 
+  PRINT_MACRO(_FASTMD); 
+  PRINT_MACRO(_MAXVL); 
+
   // misc compilers not covered so far:
   PRINT_MACRO(__USLC__);
   PRINT_MACRO(__DECCXX);
@@ -303,6 +325,8 @@ void print_compiler_macros()
   PRINT_MACRO(__QNXNTO__);
   PRINT_MACRO(__QNX__);
   PRINT_MACRO(_NTO_VERSION);
+  PRINT_MACRO(__OPEN64__);
+  PRINT_MACRO(__open64);
 }
 
 void print_stdlib_macros()
@@ -790,6 +814,8 @@ void print_platform_macros()
    PRINT_MACRO(_THREAD_SAFE);
    PRINT_MACRO(_XOPEN_SOURCE_EXTENDED);
    PRINT_MACRO(XPG);
+   PRINT_MACRO(__MINGW32_MAJOR_VERSION);
+   PRINT_MACRO(__MINGW32_MINOR_VERSION);
    // POSIX:
    PRINT_MACRO(_POSIX_ADVISORY_INFO);
    PRINT_MACRO(_POSIX_ASYNCHRONOUS_IO);
@@ -896,6 +922,7 @@ void print_boost_macros()
    PRINT_MACRO(BOOST_DISABLE_THREADS);
    PRINT_MACRO(BOOST_DISABLE_WIN32);
    PRINT_MACRO(BOOST_HAS_THREADS);
+   PRINT_MACRO(BOOST_WINDOWS);
 
    // BEGIN GENERATED BLOCK DO NOT EDIT THIS!!!!!!
    PRINT_MACRO(BOOST_BCB_PARTIAL_SPECIALIZATION_BUG);
@@ -903,10 +930,10 @@ void print_boost_macros()
    PRINT_MACRO(BOOST_FUNCTION_SCOPE_USING_DECLARATION_BREAKS_ADL);
    PRINT_MACRO(BOOST_HAS_BETHREADS);
    PRINT_MACRO(BOOST_HAS_CLOCK_GETTIME);
-   PRINT_MACRO(BOOST_HAS_CONCEPTS);
    PRINT_MACRO(BOOST_HAS_DIRENT_H);
    PRINT_MACRO(BOOST_HAS_EXPM1);
    PRINT_MACRO(BOOST_HAS_FTIME);
+   PRINT_MACRO(BOOST_HAS_GETSYSTEMTIMEASFILETIME);
    PRINT_MACRO(BOOST_HAS_GETTIMEOFDAY);
    PRINT_MACRO(BOOST_HAS_HASH);
    PRINT_MACRO(BOOST_HAS_LOG1P);
@@ -955,28 +982,66 @@ void print_boost_macros()
    PRINT_MACRO(BOOST_NO_ADL_BARRIER);
    PRINT_MACRO(BOOST_NO_ARGUMENT_DEPENDENT_LOOKUP);
    PRINT_MACRO(BOOST_NO_ARRAY_TYPE_SPECIALIZATIONS);
+   PRINT_MACRO(BOOST_NO_AUTO_DECLARATIONS);
+   PRINT_MACRO(BOOST_NO_AUTO_MULTIDECLARATIONS);
    PRINT_MACRO(BOOST_NO_AUTO_PTR);
+   PRINT_MACRO(BOOST_NO_CHAR16_T);
+   PRINT_MACRO(BOOST_NO_CHAR32_T);
+   PRINT_MACRO(BOOST_NO_COMPLETE_VALUE_INITIALIZATION);
+   PRINT_MACRO(BOOST_NO_CONSTEXPR);
    PRINT_MACRO(BOOST_NO_CTYPE_FUNCTIONS);
    PRINT_MACRO(BOOST_NO_CV_SPECIALIZATIONS);
    PRINT_MACRO(BOOST_NO_CV_VOID_SPECIALIZATIONS);
    PRINT_MACRO(BOOST_NO_CWCHAR);
    PRINT_MACRO(BOOST_NO_CWCTYPE);
+   PRINT_MACRO(BOOST_NO_CXX11_ALLOCATOR);
+   PRINT_MACRO(BOOST_NO_CXX11_ATOMIC_SMART_PTR);
+   PRINT_MACRO(BOOST_NO_CXX11_HDR_ARRAY);
+   PRINT_MACRO(BOOST_NO_CXX11_HDR_CHRONO);
+   PRINT_MACRO(BOOST_NO_CXX11_HDR_CODECVT);
+   PRINT_MACRO(BOOST_NO_CXX11_HDR_CONDITION_VARIABLE);
+   PRINT_MACRO(BOOST_NO_CXX11_HDR_FORWARD_LIST);
+   PRINT_MACRO(BOOST_NO_CXX11_HDR_FUNCTIONAL);
+   PRINT_MACRO(BOOST_NO_CXX11_HDR_FUTURE);
+   PRINT_MACRO(BOOST_NO_CXX11_HDR_INITIALIZER_LIST);
+   PRINT_MACRO(BOOST_NO_CXX11_HDR_MUTEX);
+   PRINT_MACRO(BOOST_NO_CXX11_HDR_RANDOM);
+   PRINT_MACRO(BOOST_NO_CXX11_HDR_RATIO);
+   PRINT_MACRO(BOOST_NO_CXX11_HDR_REGEX);
+   PRINT_MACRO(BOOST_NO_CXX11_HDR_SYSTEM_ERROR);
+   PRINT_MACRO(BOOST_NO_CXX11_HDR_THREAD);
+   PRINT_MACRO(BOOST_NO_CXX11_HDR_TUPLE);
+   PRINT_MACRO(BOOST_NO_CXX11_HDR_TYPEINDEX);
+   PRINT_MACRO(BOOST_NO_CXX11_HDR_TYPE_TRAITS);
+   PRINT_MACRO(BOOST_NO_CXX11_HDR_UNORDERED_MAP);
+   PRINT_MACRO(BOOST_NO_CXX11_HDR_UNORDERED_SET);
+   PRINT_MACRO(BOOST_NO_CXX11_SMART_PTR);
+   PRINT_MACRO(BOOST_NO_DECLTYPE);
+   PRINT_MACRO(BOOST_NO_DECLTYPE_N3276);
+   PRINT_MACRO(BOOST_NO_DEFAULTED_FUNCTIONS);
+   PRINT_MACRO(BOOST_NO_DELETED_FUNCTIONS);
    PRINT_MACRO(BOOST_NO_DEPENDENT_NESTED_DERIVATIONS);
    PRINT_MACRO(BOOST_NO_DEPENDENT_TYPES_IN_TEMPLATE_VALUE_PARAMETERS);
    PRINT_MACRO(BOOST_NO_EXCEPTIONS);
    PRINT_MACRO(BOOST_NO_EXCEPTION_STD_NAMESPACE);
+   PRINT_MACRO(BOOST_NO_EXPLICIT_CONVERSION_OPERATORS);
    PRINT_MACRO(BOOST_NO_EXPLICIT_FUNCTION_TEMPLATE_ARGUMENTS);
+   PRINT_MACRO(BOOST_NO_EXTERN_TEMPLATE);
+   PRINT_MACRO(BOOST_NO_FENV_H);
+   PRINT_MACRO(BOOST_NO_FUNCTION_TEMPLATE_DEFAULT_ARGS);
    PRINT_MACRO(BOOST_NO_FUNCTION_TEMPLATE_ORDERING);
    PRINT_MACRO(BOOST_NO_FUNCTION_TYPE_SPECIALIZATIONS);
    PRINT_MACRO(BOOST_NO_INCLASS_MEMBER_INITIALIZATION);
-   PRINT_MACRO(BOOST_NO_INITIALIZER_LISTS);
    PRINT_MACRO(BOOST_NO_INTEGRAL_INT64_T);
    PRINT_MACRO(BOOST_NO_INTRINSIC_WCHAR_T);
    PRINT_MACRO(BOOST_NO_IOSFWD);
    PRINT_MACRO(BOOST_NO_IOSTREAM);
    PRINT_MACRO(BOOST_NO_IS_ABSTRACT);
+   PRINT_MACRO(BOOST_NO_LAMBDAS);
    PRINT_MACRO(BOOST_NO_LIMITS);
    PRINT_MACRO(BOOST_NO_LIMITS_COMPILE_TIME_CONSTANTS);
+   PRINT_MACRO(BOOST_NO_LOCAL_CLASS_TEMPLATE_PARAMETERS);
+   PRINT_MACRO(BOOST_NO_LONG_LONG);
    PRINT_MACRO(BOOST_NO_LONG_LONG_NUMERIC_LIMITS);
    PRINT_MACRO(BOOST_NO_MEMBER_FUNCTION_SPECIALIZATIONS);
    PRINT_MACRO(BOOST_NO_MEMBER_TEMPLATES);
@@ -984,13 +1049,21 @@ void print_boost_macros()
    PRINT_MACRO(BOOST_NO_MEMBER_TEMPLATE_KEYWORD);
    PRINT_MACRO(BOOST_NO_MS_INT64_NUMERIC_LIMITS);
    PRINT_MACRO(BOOST_NO_NESTED_FRIENDSHIP);
+   PRINT_MACRO(BOOST_NO_NOEXCEPT);
+   PRINT_MACRO(BOOST_NO_NULLPTR);
+   PRINT_MACRO(BOOST_NO_NUMERIC_LIMITS_LOWEST);
    PRINT_MACRO(BOOST_NO_OPERATORS_IN_NAMESPACE);
    PRINT_MACRO(BOOST_NO_PARTIAL_SPECIALIZATION_IMPLICIT_DEFAULT_ARGS);
    PRINT_MACRO(BOOST_NO_POINTER_TO_MEMBER_CONST);
    PRINT_MACRO(BOOST_NO_POINTER_TO_MEMBER_TEMPLATE_PARAMETERS);
    PRINT_MACRO(BOOST_NO_PRIVATE_IN_AGGREGATE);
+   PRINT_MACRO(BOOST_NO_RAW_LITERALS);
    PRINT_MACRO(BOOST_NO_RTTI);
+   PRINT_MACRO(BOOST_NO_RVALUE_REFERENCES);
+   PRINT_MACRO(BOOST_NO_SCOPED_ENUMS);
    PRINT_MACRO(BOOST_NO_SFINAE);
+   PRINT_MACRO(BOOST_NO_SFINAE_EXPR);
+   PRINT_MACRO(BOOST_NO_STATIC_ASSERT);
    PRINT_MACRO(BOOST_NO_STDC_NAMESPACE);
    PRINT_MACRO(BOOST_NO_STD_ALLOCATOR);
    PRINT_MACRO(BOOST_NO_STD_DISTANCE);
@@ -1009,14 +1082,19 @@ void print_boost_macros()
    PRINT_MACRO(BOOST_NO_SWPRINTF);
    PRINT_MACRO(BOOST_NO_TEMPLATED_IOSTREAMS);
    PRINT_MACRO(BOOST_NO_TEMPLATED_ITERATOR_CONSTRUCTORS);
+   PRINT_MACRO(BOOST_NO_TEMPLATE_ALIASES);
    PRINT_MACRO(BOOST_NO_TEMPLATE_PARTIAL_SPECIALIZATION);
    PRINT_MACRO(BOOST_NO_TEMPLATE_TEMPLATES);
    PRINT_MACRO(BOOST_NO_TWO_PHASE_NAME_LOOKUP);
    PRINT_MACRO(BOOST_NO_TYPEID);
    PRINT_MACRO(BOOST_NO_TYPENAME_WITH_CTOR);
+   PRINT_MACRO(BOOST_NO_UNICODE_LITERALS);
+   PRINT_MACRO(BOOST_NO_UNIFIED_INITIALIZATION_SYNTAX);
    PRINT_MACRO(BOOST_NO_UNREACHABLE_RETURN_DETECTION);
    PRINT_MACRO(BOOST_NO_USING_DECLARATION_OVERLOADS_FROM_TYPENAME_BASE);
    PRINT_MACRO(BOOST_NO_USING_TEMPLATE);
+   PRINT_MACRO(BOOST_NO_VARIADIC_MACROS);
+   PRINT_MACRO(BOOST_NO_VARIADIC_TEMPLATES);
    PRINT_MACRO(BOOST_NO_VOID_RETURNS);
 
 
@@ -1024,16 +1102,6 @@ void print_boost_macros()
 
 
 
-
-
-
-
-
-
-
-
-
-
 
 
 
@@ -1051,6 +1119,11 @@ void print_boost_macros()
    PRINT_MACRO(BOOST_MSVC);
    PRINT_MACRO(BOOST_STD_EXTENSION_NAMESPACE);
    PRINT_MACRO(BOOST_UNREACHABLE_RETURN(0));
+   PRINT_MACRO(BOOST_CONSTEXPR);
+   PRINT_MACRO(BOOST_CONSTEXPR_OR_CONST);
+   PRINT_MACRO(BOOST_STATIC_CONSTEXPR);
+   PRINT_MACRO(BOOST_NOEXCEPT);
+   PRINT_MACRO(BOOST_FORCEINLINE);
 }
 
 void print_separator()
@@ -1073,14 +1146,3 @@ int main()
 
   return 0;
 }
-
-
-
-
-
-
-
-
-
-
-
