@@ -4,7 +4,7 @@
  * Boost Software License, Version 1.0. (See accompanying
  * file LICENSE_1_0.txt or http://www.boost.org/LICENSE_1_0.txt)
  * Author: Jeff Garland, Bart Garst
- * $Date: 2009-02-01 06:29:43 -0500 (Sun, 01 Feb 2009) $
+ * $Date: 2012-05-26 15:33:09 -0400 (Sat, 26 May 2012) $
  */
 
 #include "boost/date_time/posix_time/posix_time.hpp"
@@ -108,6 +108,25 @@ int main() {
       ss << long_td;
       check("Long time durations", ss.str() == std::string("300:02:01"));
       ss.str("");
+
+      // Short-hand format specifiers
+      f->format("%T");
+      f->time_duration_format("%T");
+      ss << t;
+      check("Short-hand '%T' in time format", ss.str() == std::string("18:01:56"));
+      ss.str("");
+      ss << td;
+      check("Short-hand '%T' in time_duration format", ss.str() == std::string("03:02:01"));
+      ss.str("");
+
+      f->format("%R");
+      f->time_duration_format("%R");
+      ss << t;
+      check("Short-hand '%R' in time format", ss.str() == std::string("18:01"));
+      ss.str("");
+      ss << td;
+      check("Short-hand '%R' in time_duration format", ss.str() == std::string("03:02"));
+      ss.str("");
     }
     { // negative time_duration tests
       std::string result;
@@ -146,7 +165,7 @@ int main() {
       //reset the sign to always print
       time_facet* f = new time_facet();
       ss.imbue(std::locale(ss.getloc(), f));
-      f->time_duration_format("%+%H:%M:%S%F");
+      f->time_duration_format("%+%H:%M:%S""%F");
 
       ss.str("");
       ss << td4 - td3;
@@ -329,20 +348,20 @@ int main() {
     }
 
     {
-      wtime_facet* wtimefacet = new wtime_facet(L"%Y-%b-%d %H:%M:%S%F");
-      teststreaming("widestream custom time no frac seconds %F operator: %Y-%b-%d %H:%M:%S%F", t, 
+      wtime_facet* wtimefacet = new wtime_facet(L"%Y-%b-%d %H:%M:%S" L"%F");
+      teststreaming("widestream custom time no frac seconds %F operator: %Y-%b-%d %H:%M:%S""%F", t, 
                     std::wstring(L"2004-Oct-13 18:01:56"), 
                     std::locale(std::locale::classic(), wtimefacet));
     }
 
     {
-      wtime_facet* wtimefacet = new wtime_facet(L"%Y-%b-%d %H:%M:%S%F");
+      wtime_facet* wtimefacet = new wtime_facet(L"%Y-%b-%d %H:%M:%S" L"%F");
 #ifdef BOOST_DATE_TIME_HAS_NANOSECONDS
-      teststreaming("widestream custom time with frac seconds %F operator: %Y-%b-%d %H:%M:%S%F", tf, 
+      teststreaming("widestream custom time with frac seconds %F operator: %Y-%b-%d %H:%M:%S""%F", tf, 
                     std::wstring(L"2004-Oct-13 18:01:56.000003000"), 
                     std::locale(std::locale::classic(), wtimefacet));
 #else
-      teststreaming("widestream custom time with frac seconds %F operator: %Y-%b-%d %H:%M:%S%F", tf, 
+      teststreaming("widestream custom time with frac seconds %F operator: %Y-%b-%d %H:%M:%S""%F", tf, 
                     std::wstring(L"2004-Oct-13 18:01:56.000003"), 
                     std::locale(std::locale::classic(), wtimefacet));
 #endif // BOOST_DATE_TIME_HAS_NANOSECONDS
@@ -376,13 +395,13 @@ int main() {
 
 
     {
-      wtime_facet* wtimefacet = new wtime_facet(L"%Y-%b-%d %H:%M:%S%F");
+      wtime_facet* wtimefacet = new wtime_facet(L"%Y-%b-%d %H:%M:%S" L"%F");
 #ifdef BOOST_DATE_TIME_HAS_NANOSECONDS
-      teststreaming("widestream time period frac seconds %F operator: %Y-%b-%d %H:%M:%S%F", tp, 
+      teststreaming("widestream time period frac seconds %F operator: %Y-%b-%d %H:%M:%S""%F", tp, 
                     std::wstring(L"[2004-Oct-13 18:01:56/2004-Oct-20 19:02:57.000002999]"), 
                     std::locale(std::locale::classic(), wtimefacet));
 #else
-      teststreaming("widestream time period frac seconds %F operator: %Y-%b-%d %H:%M:%S%F", tp, 
+      teststreaming("widestream time period frac seconds %F operator: %Y-%b-%d %H:%M:%S""%F", tp, 
                     std::wstring(L"[2004-Oct-13 18:01:56/2004-Oct-20 19:02:57.000002]"), 
                     std::locale(std::locale::classic(), wtimefacet));
 #endif // BOOST_DATE_TIME_HAS_NANOSECONDS
