@@ -38,12 +38,12 @@ namespace boost {
 namespace interprocess {
 namespace test {
 
-//!An STL compatible heap_allocator_v1 that uses a segment manager as 
+//!An STL compatible heap_allocator_v1 that uses a segment manager as
 //!memory source. The internal pointer type will of the same type (raw, smart) as
 //!"typename SegmentManager::void_pointer" type. This allows
 //!placing the heap_allocator_v1 in shared memory, memory mapped-files, etc...*/
 template<class T, class SegmentManager>
-class heap_allocator_v1 
+class heap_allocator_v1
 {
  private:
    typedef heap_allocator_v1<T, SegmentManager>         self_t;
@@ -83,14 +83,14 @@ class heap_allocator_v1
    //!Obtains an heap_allocator_v1 of other type
    template<class T2>
    struct rebind
-   {   
+   {  
       typedef heap_allocator_v1<T2, SegmentManager>     other;
    };
 
    //!Returns the segment manager. Never throws
    segment_manager* get_segment_manager()const
    {  return ipcdetail::to_raw_pointer(mp_mngr);   }
-/*
+
    //!Returns address of mutable object. Never throws
    pointer address(reference value) const
    {  return pointer(addressof(value));  }
@@ -98,30 +98,34 @@ class heap_allocator_v1
    //!Returns address of non mutable object. Never throws
    const_pointer address(const_reference value) const
    {  return const_pointer(addressof(value));  }
-*/
+
    //!Constructor from the segment manager. Never throws
-   heap_allocator_v1(segment_manager *segment_mngr) 
+   heap_allocator_v1(segment_manager *segment_mngr)
       : mp_mngr(segment_mngr) { }
 
    //!Constructor from other heap_allocator_v1. Never throws
-   heap_allocator_v1(const heap_allocator_v1 &other) 
+   heap_allocator_v1(const heap_allocator_v1 &other)
       : mp_mngr(other.get_segment_manager()){ }
 
    //!Constructor from related heap_allocator_v1. Never throws
    template<class T2>
-   heap_allocator_v1(const heap_allocator_v1<T2, SegmentManager> &other) 
+   heap_allocator_v1(const heap_allocator_v1<T2, SegmentManager> &other)
       : mp_mngr(other.get_segment_manager()){}
 
-   //!Allocates memory for an array of count elements. 
+   //!Allocates memory for an array of count elements.
    //!Throws boost::interprocess::bad_alloc if there is no enough memory
    pointer allocate(size_type count, cvoid_ptr hint = 0)
-   {  (void)hint; return ::new value_type[count];  }
+   {
+      (void)hint;
+      char *raw_mem = ::new char[sizeof(value_type)*count];
+      return boost::intrusive::pointer_traits<pointer>::pointer_to(reinterpret_cast<value_type &>(*raw_mem));
+   }
 
    //!Deallocates memory previously allocated. Never throws
    void deallocate(const pointer &ptr, size_type)
    {  return ::delete[] ipcdetail::to_raw_pointer(ptr) ;  }
 
-   //!Construct object, calling constructor. 
+   //!Construct object, calling constructor.
    //!Throws if T(const T&) throws
    void construct(const pointer &ptr, const_reference value)
    {  new((void*)ipcdetail::to_raw_pointer(ptr)) value_type(value);  }
@@ -142,13 +146,13 @@ class heap_allocator_v1
 
 //!Equality test for same type of heap_allocator_v1
 template<class T, class SegmentManager> inline
-bool operator==(const heap_allocator_v1<T , SegmentManager>  &alloc1, 
+bool operator==(const heap_allocator_v1<T , SegmentManager>  &alloc1,
                 const heap_allocator_v1<T, SegmentManager>  &alloc2)
    {  return alloc1.get_segment_manager() == alloc2.get_segment_manager(); }
 
 //!Inequality test for same type of heap_allocator_v1
 template<class T, class SegmentManager> inline
-bool operator!=(const heap_allocator_v1<T, SegmentManager>  &alloc1, 
+bool operator!=(const heap_allocator_v1<T, SegmentManager>  &alloc1,
                 const heap_allocator_v1<T, SegmentManager>  &alloc2)
    {  return alloc1.get_segment_manager() != alloc2.get_segment_manager(); }
 
