@@ -1,6 +1,6 @@
 //////////////////////////////////////////////////////////////////////////////
 //
-// (C) Copyright Ion Gaztanaga 2004-2007. Distributed under the Boost
+// (C) Copyright Ion Gaztanaga 2004-2011. Distributed under the Boost
 // Software License, Version 1.0. (See accompanying file
 // LICENSE_1_0.txt or copy at http://www.boost.org/LICENSE_1_0.txt)
 //
@@ -39,7 +39,7 @@ int main ()
       managed_shared_memory shmem(create_only, ShmemName, ShmemSize);
 
       int i;
-      //Let's allocate some memory 
+      //Let's allocate some memory
       for(i = 0; i < max; ++i){
          array[i] = shmem.allocate(i+1);
       }
@@ -122,8 +122,9 @@ int main ()
       if(!shmem_vect)
          return -1;
    }
+   #ifndef BOOST_INTERPROCESS_POSIX_SHARED_MEMORY_OBJECTS_NO_GROW
    {
-      std::size_t old_free_memory;
+      managed_shared_memory::size_type old_free_memory;
       {
          //Map preexisting shmem again in memory
          managed_shared_memory shmem(open_only, ShmemName);
@@ -147,7 +148,7 @@ int main ()
          return -1;
    }
    {
-      std::size_t old_free_memory, next_free_memory,
+      managed_shared_memory::size_type old_free_memory, next_free_memory,
                   old_shmem_size, next_shmem_size, final_shmem_size;
       {
          //Map preexisting shmem again in memory
@@ -196,15 +197,16 @@ int main ()
          if(next_shmem_size <= final_shmem_size)
             return -1;
       }
-      {
-         //Now test move semantics
-         managed_shared_memory original(open_only, ShmemName);
-         managed_shared_memory move_ctor(detail::move_impl(original));
-         managed_shared_memory move_assign;
-         move_assign = detail::move_impl(move_ctor);
-         move_assign.swap(detail::move_impl(original));
-         move_assign.swap(original);
-      }
+   }
+   #endif //ifndef BOOST_INTERPROCESS_POSIX_SHARED_MEMORY_OBJECTS_NO_GROW
+
+   {
+      //Now test move semantics
+      managed_shared_memory original(open_only, ShmemName);
+      managed_shared_memory move_ctor(boost::move(original));
+      managed_shared_memory move_assign;
+      move_assign = boost::move(move_ctor);
+      move_assign.swap(original);
    }
 
    shared_memory_object::remove(ShmemName);
