@@ -1,7 +1,7 @@
 /////////////////////////////////////////////////////////////////////////////
 //
 // (C) Copyright Olaf Krzikalla 2004-2006.
-// (C) Copyright Ion Gaztanaga  2006-2008.
+// (C) Copyright Ion Gaztanaga  2006-2009.
 //
 // Distributed under the Boost Software License, Version 1.0.
 //    (See accompanying file LICENSE_1_0.txt or copy at
@@ -12,7 +12,7 @@
 /////////////////////////////////////////////////////////////////////////////
 #include <boost/intrusive/detail/config_begin.hpp>
 #include <boost/intrusive/unordered_set.hpp>
-#include <boost/intrusive/detail/pointer_to_other.hpp>
+#include <boost/intrusive/pointer_traits.hpp>
 #include "itestvalue.hpp"
 #include "smart_ptr.hpp"
 #include "common_functors.hpp"
@@ -22,6 +22,26 @@
 #include "test_macros.hpp"
 #include "test_container.hpp"
 
+namespace boost { namespace intrusive { namespace test {
+
+#if !defined (BOOST_INTRUSIVE_VARIADIC_TEMPLATES)
+template<class T, class O1, class O2, class O3, class O4, class O5, class O6>
+#else
+template<class T, class ...Options>
+#endif
+struct is_unordered<boost::intrusive::unordered_set<T,
+   #if !defined (BOOST_INTRUSIVE_VARIADIC_TEMPLATES)
+   O1, O2, O3, O4, O5, O6
+   #else
+   Options...
+   #endif
+> >
+{
+   static const bool value = true;
+};
+
+}}}
+
 using namespace boost::intrusive;
 
 struct my_tag;
@@ -51,7 +71,7 @@ struct hooks
 static const std::size_t BucketSize = 8;
 
 template<class ValueTraits, bool CacheBegin, bool CompareHash, bool Incremental>
-struct test_unordered_set 
+struct test_unordered_set
 {
    typedef typename ValueTraits::value_type value_type;
    static void test_all(std::vector<value_type>& values);
@@ -81,7 +101,9 @@ void test_unordered_set<ValueTraits, CacheBegin, CompareHash, Incremental>::
    typedef typename unordered_set_type::bucket_traits bucket_traits;
    {
       typename unordered_set_type::bucket_type buckets [BucketSize];
-      unordered_set_type testset(bucket_traits(buckets, BucketSize));
+      unordered_set_type testset(bucket_traits(
+         pointer_traits<typename unordered_set_type::bucket_ptr>::
+            pointer_to(buckets[0]), BucketSize));
       testset.insert(values.begin(), values.end());
       test::test_container(testset);
       testset.clear();
@@ -120,10 +142,12 @@ void test_unordered_set<ValueTraits, CacheBegin, CompareHash, Incremental>::test
 
    std::vector<value_type> values (5);
    for (int i = 0; i < 5; ++i)
-      values[i].value_ = i; 
+      values[i].value_ = i;
 
    typename unordered_set_type::bucket_type buckets [BucketSize];
-   unordered_set_type testset(bucket_traits(buckets, BucketSize));
+   unordered_set_type testset(bucket_traits(
+      pointer_traits<typename unordered_set_type::bucket_ptr>::
+            pointer_to(buckets[0]), BucketSize));
    for (int i = 0; i < 5; ++i)
       testset.insert (values[i]);
 
@@ -151,7 +175,9 @@ void test_unordered_set<ValueTraits, CacheBegin, CompareHash, Incremental>::
    typedef typename unordered_set_type::bucket_traits bucket_traits;
 
    typename unordered_set_type::bucket_type buckets [BucketSize];
-   unordered_set_type testset1(values.begin(), values.end(), bucket_traits(buckets, BucketSize));
+   unordered_set_type testset1(values.begin(), values.end(), bucket_traits(
+      pointer_traits<typename unordered_set_type::bucket_ptr>::
+         pointer_to(buckets[0]), BucketSize));
    BOOST_TEST (5 == std::distance(testset1.begin(), testset1.end()));
 
    if(Incremental){
@@ -162,11 +188,11 @@ void test_unordered_set<ValueTraits, CacheBegin, CompareHash, Incremental>::
       {  int init_values [] = { 1, 2, 3, 4, 5 };
          TEST_INTRUSIVE_SEQUENCE( init_values, testset1.begin() );  }
    }
-    
+   
    testset1.clear();
    BOOST_TEST (testset1.empty());
-}  
-  
+} 
+ 
 //test: insert, const_iterator, const_reverse_iterator, erase, iterator_to:
 template<class ValueTraits, bool CacheBegin, bool CompareHash, bool Incremental>
 void test_unordered_set<ValueTraits, CacheBegin, CompareHash, Incremental>::
@@ -184,7 +210,9 @@ void test_unordered_set<ValueTraits, CacheBegin, CompareHash, Incremental>::
    typedef typename unordered_set_type::bucket_traits bucket_traits;
 
    typename unordered_set_type::bucket_type buckets [BucketSize];
-   unordered_set_type testset(bucket_traits(buckets, BucketSize));
+   unordered_set_type testset(bucket_traits(
+      pointer_traits<typename unordered_set_type::bucket_ptr>::
+         pointer_to(buckets[0]), BucketSize));
    testset.insert(&values[0] + 2, &values[0] + 5);
 
    const unordered_set_type& const_testset = testset;
@@ -223,7 +251,7 @@ void test_unordered_set<ValueTraits, CacheBegin, CompareHash, Incremental>::
       {  int init_values [] = { 1, 3, 5 };
          TEST_INTRUSIVE_SEQUENCE( init_values, const_testset.begin() );  }
    }
-}  
+} 
 
 //test: insert (seq-version), swap, erase (seq-version), size:
 template<class ValueTraits, bool CacheBegin, bool CompareHash, bool Incremental>
@@ -243,8 +271,12 @@ void test_unordered_set<ValueTraits, CacheBegin, CompareHash, Incremental>::
 
    typename unordered_set_type::bucket_type buckets1 [BucketSize];
    typename unordered_set_type::bucket_type buckets2 [BucketSize];
-   unordered_set_type testset1(&values[0], &values[0] + 2, bucket_traits(buckets1, BucketSize));
-   unordered_set_type testset2(bucket_traits(buckets2, BucketSize));
+   unordered_set_type testset1(&values[0], &values[0] + 2, bucket_traits(
+      pointer_traits<typename unordered_set_type::bucket_ptr>::
+         pointer_to(buckets1[0]), BucketSize));
+   unordered_set_type testset2(bucket_traits(
+      pointer_traits<typename unordered_set_type::bucket_ptr>::
+         pointer_to(buckets2[0]), BucketSize));
 
    testset2.insert (&values[0] + 2, &values[0] + 6);
    testset1.swap (testset2);
@@ -267,7 +299,7 @@ void test_unordered_set<ValueTraits, CacheBegin, CompareHash, Incremental>::
       BOOST_TEST (testset1.size() == 1);
       BOOST_TEST (&*testset1.begin() == &values[3]);
    }
-}  
+} 
 
 //test: rehash:
 template<class ValueTraits, bool CacheBegin, bool CompareHash, bool Incremental>
@@ -287,7 +319,9 @@ void test_unordered_set<ValueTraits, CacheBegin, CompareHash, Incremental>::
    //Build a uset
    typename unordered_set_type::bucket_type buckets1 [BucketSize];
    typename unordered_set_type::bucket_type buckets2 [BucketSize*2];
-   unordered_set_type testset1(&values[0], &values[0] + 6, bucket_traits(buckets1, BucketSize));
+   unordered_set_type testset1(&values[0], &values[0] + 6, bucket_traits(
+      pointer_traits<typename unordered_set_type::bucket_ptr>::
+         pointer_to(buckets1[0]), BucketSize));
    //Test current state
    BOOST_TEST(testset1.split_count() == BucketSize/2);
    {  int init_values [] = { 4, 5, 1, 2, 3 };
@@ -314,21 +348,27 @@ void test_unordered_set<ValueTraits, CacheBegin, CompareHash, Incremental>::
    //Try incremental hashing specifying a new bucket traits pointing to the same array
    //
    //This incremental rehash should fail because the new size is not twice the original
-   BOOST_TEST(testset1.incremental_rehash(bucket_traits(buckets1, BucketSize)) == false);
+   BOOST_TEST(testset1.incremental_rehash(bucket_traits(
+      pointer_traits<typename unordered_set_type::bucket_ptr>::
+         pointer_to(buckets1[0]), BucketSize)) == false);
    BOOST_TEST(testset1.split_count() == BucketSize);
    {  int init_values [] = { 1, 2, 3, 4, 5 };
    TEST_INTRUSIVE_SEQUENCE( init_values, testset1.begin() );  }
 
    //This incremental rehash should success because the new size is twice the original
    //and split_count is the same as the old bucket count
-   BOOST_TEST(testset1.incremental_rehash(bucket_traits(buckets1, BucketSize*2)) == true);
+   BOOST_TEST(testset1.incremental_rehash(bucket_traits(
+      pointer_traits<typename unordered_set_type::bucket_ptr>::
+         pointer_to(buckets1[0]), BucketSize*2)) == true);
    BOOST_TEST(testset1.split_count() == BucketSize);
    {  int init_values [] = { 1, 2, 3, 4, 5 };
    TEST_INTRUSIVE_SEQUENCE( init_values, testset1.begin() );  }
 
    //This incremental rehash should also success because the new size is half the original
    //and split_count is the same as the new bucket count
-   BOOST_TEST(testset1.incremental_rehash(bucket_traits(buckets1, BucketSize)) == true);
+   BOOST_TEST(testset1.incremental_rehash(bucket_traits(
+      pointer_traits<typename unordered_set_type::bucket_ptr>::
+         pointer_to(buckets1[0]), BucketSize)) == true);
    BOOST_TEST(testset1.split_count() == BucketSize);
    {  int init_values [] = { 1, 2, 3, 4, 5 };
    TEST_INTRUSIVE_SEQUENCE( init_values, testset1.begin() );  }
@@ -337,39 +377,51 @@ void test_unordered_set<ValueTraits, CacheBegin, CompareHash, Incremental>::
    //Try incremental hashing specifying a new bucket traits pointing to the same array
    //
    //This incremental rehash should fail because the new size is not twice the original
-   BOOST_TEST(testset1.incremental_rehash(bucket_traits(buckets2, BucketSize)) == false);
+   BOOST_TEST(testset1.incremental_rehash(bucket_traits(
+      pointer_traits<typename unordered_set_type::bucket_ptr>::
+         pointer_to(buckets2[0]), BucketSize)) == false);
    BOOST_TEST(testset1.split_count() == BucketSize);
    {  int init_values [] = { 1, 2, 3, 4, 5 };
    TEST_INTRUSIVE_SEQUENCE( init_values, testset1.begin() );  }
 
    //This incremental rehash should success because the new size is twice the original
    //and split_count is the same as the old bucket count
-   BOOST_TEST(testset1.incremental_rehash(bucket_traits(buckets2, BucketSize*2)) == true);
+   BOOST_TEST(testset1.incremental_rehash(bucket_traits(
+      pointer_traits<typename unordered_set_type::bucket_ptr>::
+         pointer_to(buckets2[0]), BucketSize*2)) == true);
    BOOST_TEST(testset1.split_count() == BucketSize);
    {  int init_values [] = { 1, 2, 3, 4, 5 };
    TEST_INTRUSIVE_SEQUENCE( init_values, testset1.begin() );  }
 
    //This incremental rehash should also success because the new size is half the original
    //and split_count is the same as the new bucket count
-   BOOST_TEST(testset1.incremental_rehash(bucket_traits(buckets1, BucketSize)) == true);
+   BOOST_TEST(testset1.incremental_rehash(bucket_traits(
+      pointer_traits<typename unordered_set_type::bucket_ptr>::
+         pointer_to(buckets1[0]), BucketSize)) == true);
    BOOST_TEST(testset1.split_count() == BucketSize);
    {  int init_values [] = { 1, 2, 3, 4, 5 };
    TEST_INTRUSIVE_SEQUENCE( init_values, testset1.begin() );  }
 
    //Full shrink rehash
-   testset1.rehash(bucket_traits(buckets1, 4));
+   testset1.rehash(bucket_traits(
+      pointer_traits<typename unordered_set_type::bucket_ptr>::
+         pointer_to(buckets1[0]), 4));
    BOOST_TEST (testset1.size() == values.size()-1);
    BOOST_TEST (testset1.incremental_rehash() == false);
    {  int init_values [] = { 4, 5, 1, 2, 3 };
       TEST_INTRUSIVE_SEQUENCE( init_values, testset1.begin() );  }
    //Full shrink rehash again
-   testset1.rehash(bucket_traits(buckets1, 2));
+   testset1.rehash(bucket_traits(
+      pointer_traits<typename unordered_set_type::bucket_ptr>::
+         pointer_to(buckets1[0]), 2));
    BOOST_TEST (testset1.size() == values.size()-1);
    BOOST_TEST (testset1.incremental_rehash() == false);
    {  int init_values [] = { 2, 4, 3, 5, 1 };
       TEST_INTRUSIVE_SEQUENCE( init_values, testset1.begin() );  }
    //Full growing rehash
-   testset1.rehash(bucket_traits(buckets1, BucketSize));
+   testset1.rehash(bucket_traits(
+      pointer_traits<typename unordered_set_type::bucket_ptr>::
+         pointer_to(buckets1[0]), BucketSize));
    BOOST_TEST (testset1.size() == values.size()-1);
    BOOST_TEST (testset1.incremental_rehash() == false);
    {  int init_values [] = { 1, 2, 3, 4, 5 };
@@ -419,33 +471,43 @@ void test_unordered_set<ValueTraits, CacheBegin, CompareHash, Incremental>::
    typename unordered_set_type::bucket_type buckets2 [2];
    typename unordered_set_type::bucket_type buckets3 [BucketSize*2];
 
-   unordered_set_type testset1(&values[0], &values[0] + 6, bucket_traits(buckets1, BucketSize));
+   unordered_set_type testset1(&values[0], &values[0] + 6, bucket_traits(
+      pointer_traits<typename unordered_set_type::bucket_ptr>::
+         pointer_to(buckets1[0]), BucketSize));
    BOOST_TEST (testset1.size() == values.size()-1);
    {  int init_values [] = { 1, 2, 3, 4, 5 };
       TEST_INTRUSIVE_SEQUENCE( init_values, testset1.begin() );  }
 
-   testset1.rehash(bucket_traits(buckets2, 2));
+   testset1.rehash(bucket_traits(
+      pointer_traits<typename unordered_set_type::bucket_ptr>::
+         pointer_to(buckets2[0]), 2));
    BOOST_TEST (testset1.size() == values.size()-1);
    {  int init_values [] = { 4, 2, 5, 3, 1 };
       TEST_INTRUSIVE_SEQUENCE( init_values, testset1.begin() );  }
 
-   testset1.rehash(bucket_traits(buckets3, BucketSize*2));
+   testset1.rehash(bucket_traits(
+      pointer_traits<typename unordered_set_type::bucket_ptr>::
+         pointer_to(buckets3[0]), BucketSize*2));
    BOOST_TEST (testset1.size() == values.size()-1);
    {  int init_values [] = { 1, 2, 3, 4, 5 };
       TEST_INTRUSIVE_SEQUENCE( init_values, testset1.begin() );  }
 
    //Now rehash reducing the buckets
-   testset1.rehash(bucket_traits(buckets3, 2));
+   testset1.rehash(bucket_traits(
+      pointer_traits<typename unordered_set_type::bucket_ptr>::
+         pointer_to(buckets3[0]), 2));
    BOOST_TEST (testset1.size() == values.size()-1);
    {  int init_values [] = { 4, 2, 5, 3, 1 };
       TEST_INTRUSIVE_SEQUENCE( init_values, testset1.begin() );  }
 
    //Now rehash increasing the buckets
-   testset1.rehash(bucket_traits(buckets3, BucketSize*2));
+   testset1.rehash(bucket_traits(
+      pointer_traits<typename unordered_set_type::bucket_ptr>::
+         pointer_to(buckets3[0]), BucketSize*2));
    BOOST_TEST (testset1.size() == values.size()-1);
    {  int init_values [] = { 1, 2, 3, 4, 5 };
       TEST_INTRUSIVE_SEQUENCE( init_values, testset1.begin() );  }
-}  
+} 
 
 
 //test: find, equal_range (lower_bound, upper_bound):
@@ -465,7 +527,9 @@ void test_unordered_set<ValueTraits, CacheBegin, CompareHash, Incremental>::
    typedef typename unordered_set_type::bucket_traits bucket_traits;
 
    typename unordered_set_type::bucket_type buckets [BucketSize];
-   unordered_set_type testset (values.begin(), values.end(), bucket_traits(buckets, BucketSize));
+   unordered_set_type testset (values.begin(), values.end(), bucket_traits(
+      pointer_traits<typename unordered_set_type::bucket_ptr>::
+         pointer_to(buckets[0]), BucketSize));
    typedef typename unordered_set_type::iterator iterator;
 
    value_type cmp_val;
@@ -474,7 +538,7 @@ void test_unordered_set<ValueTraits, CacheBegin, CompareHash, Incremental>::
    BOOST_TEST (i->value_ == 2);
    BOOST_TEST ((++i)->value_ != 2);
    std::pair<iterator,iterator> range = testset.equal_range (cmp_val);
-     
+    
    BOOST_TEST (range.first->value_ == 2);
    BOOST_TEST (range.second->value_ == 3);
    BOOST_TEST (std::distance (range.first, range.second) == 1);
@@ -501,8 +565,12 @@ void test_unordered_set<ValueTraits, CacheBegin, CompareHash, Incremental>
       //Test with equal bucket arrays
       typename unordered_set_type::bucket_type buckets1 [BucketSize];
       typename unordered_set_type::bucket_type buckets2 [BucketSize];
-      unordered_set_type testset1 (values.begin(), values.end(), bucket_traits(buckets1, BucketSize));
-      unordered_set_type testset2 (bucket_traits(buckets2, BucketSize));
+      unordered_set_type testset1 (values.begin(), values.end(), bucket_traits(
+         pointer_traits<typename unordered_set_type::bucket_ptr>::
+         pointer_to(buckets1[0]), BucketSize));
+      unordered_set_type testset2 (bucket_traits(
+         pointer_traits<typename unordered_set_type::bucket_ptr>::
+            pointer_to(buckets2[0]), BucketSize));
 
       testset2.clone_from(testset1, test::new_cloner<value_type>(), test::delete_disposer<value_type>());
       //Ordering is not guarantee in the cloning so insert data in a set and test
@@ -518,8 +586,12 @@ void test_unordered_set<ValueTraits, CacheBegin, CompareHash, Incremental>
       //Test with bigger source bucket arrays
       typename unordered_set_type::bucket_type buckets1 [BucketSize*2];
       typename unordered_set_type::bucket_type buckets2 [BucketSize];
-      unordered_set_type testset1 (values.begin(), values.end(), bucket_traits(buckets1, BucketSize*2));
-      unordered_set_type testset2 (bucket_traits(buckets2, BucketSize));
+      unordered_set_type testset1 (values.begin(), values.end(), bucket_traits(
+         pointer_traits<typename unordered_set_type::bucket_ptr>::
+         pointer_to(buckets1[0]), BucketSize*2));
+      unordered_set_type testset2 (bucket_traits(
+         pointer_traits<typename unordered_set_type::bucket_ptr>::
+            pointer_to(buckets2[0]), BucketSize));
 
       testset2.clone_from(testset1, test::new_cloner<value_type>(), test::delete_disposer<value_type>());
       //Ordering is not guaranteed in the cloning so insert data in a set and test
@@ -535,8 +607,12 @@ void test_unordered_set<ValueTraits, CacheBegin, CompareHash, Incremental>
       //Test with smaller source bucket arrays
       typename unordered_set_type::bucket_type buckets1 [BucketSize];
       typename unordered_set_type::bucket_type buckets2 [BucketSize*2];
-      unordered_set_type testset1 (values.begin(), values.end(), bucket_traits(buckets1, BucketSize));
-      unordered_set_type testset2 (bucket_traits(buckets2, BucketSize*2));
+      unordered_set_type testset1 (values.begin(), values.end(), bucket_traits(
+         pointer_traits<typename unordered_set_type::bucket_ptr>::
+            pointer_to(buckets1[0]), BucketSize));
+      unordered_set_type testset2 (bucket_traits(
+         pointer_traits<typename unordered_set_type::bucket_ptr>::
+            pointer_to(buckets2[0]), BucketSize*2));
 
       testset2.clone_from(testset1, test::new_cloner<value_type>(), test::delete_disposer<value_type>());
       //Ordering is not guarantee in the cloning so insert data in a set and test
@@ -560,7 +636,7 @@ class test_main_template
       static const int random_init[6] = { 3, 2, 4, 1, 5, 2 };
       std::vector<testvalue<hooks<VoidPointer> , constant_time_size> > data (6);
       for (int i = 0; i < 6; ++i)
-         data[i].value_ = random_init[i]; 
+         data[i].value_ = random_init[i];
 
       test_unordered_set < typename detail::get_base_value_traits
                   < value_type
@@ -596,7 +672,7 @@ class test_main_template<VoidPointer, false, incremental>
       static const int random_init[6] = { 3, 2, 4, 1, 5, 2 };
       std::vector<testvalue<hooks<VoidPointer> , false> > data (6);
       for (int i = 0; i < 6; ++i)
-         data[i].value_ = random_init[i]; 
+         data[i].value_ = random_init[i];
 
       test_unordered_set < typename detail::get_base_value_traits
                   < value_type
@@ -623,7 +699,7 @@ class test_main_template<VoidPointer, false, incremental>
                   < value_type
                   , typename hooks<VoidPointer>::auto_base_hook_type
                   >::type
-                , true
+                , false
                 , true
                 , incremental
                 >::test_all(data);
@@ -643,7 +719,7 @@ class test_main_template<VoidPointer, false, incremental>
    }
 };
 
-int main( int, char* [] ) 
+int main( int, char* [] )
 {
    test_main_template<void*, false, true>()();
    test_main_template<smart_ptr<void>, false, true>()();
