@@ -8,8 +8,6 @@
 
 // compile test for traits
 #include "test_tools.hpp"
-#include <boost/preprocessor/stringize.hpp>
-#include BOOST_PP_STRINGIZE(BOOST_ARCHIVE_TEST)
 
 #include <boost/serialization/level.hpp>
 #include <boost/serialization/tracking.hpp>
